﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace xReactor.Common
{
    public class Room : CommonBase
    {
        public Property<string> NameProperty;
        public string Name
        {
            get { return NameProperty.Value; }
            set { NameProperty.Value = value; }
        }

        private Property<double> sqmProperty;
        public double SquareMeters
        {
            get { return sqmProperty.Value; }
            set { sqmProperty.Value = value; }
        }

        private Property<int> peopleLimitProperty;
        public int PeopleLimit
        {
            get { return peopleLimitProperty.Value; } //read-only
        }

        private Property<ObservableCollection<Furniture>> furnitureProperty;
        public ObservableCollection<Furniture> Furniture
        {
            get { return furnitureProperty.Value; }
        }

        private Property<int> numFurnitureProperty;
        public int NumFurniture
        {
            get { return numFurnitureProperty.Value; }
        }

        private Property<Sitable> largestPlaceToSitProperty;
        public Sitable LargestPlaceToSit
        {
            get { return largestPlaceToSitProperty.Value; }
        }

        private Property<Person> ownerProperty;
        public Person Owner
        {
            get { return ownerProperty.Value; }
            set { ownerProperty.Value = value; }
        }

        private Property<int> ownerAgeProperty;
        public int OwnerAge
        {
            get { return ownerAgeProperty.Value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:Room"/> class.
        /// </summary>
        public Room()
        {
            NameProperty = Create(() => Name, "VIP room");
            sqmProperty = Create(() => SquareMeters, 0.0);
            peopleLimitProperty = Create(() => PeopleLimit, () => (int)SquareMeters / 10);
            furnitureProperty = Create(() => Furniture, new ObservableCollection<Furniture>());
            numFurnitureProperty = Create(() => NumFurniture, () => Furniture.Count);

            largestPlaceToSitProperty = Create(() => LargestPlaceToSit,
                () => Furniture.OfType<Sitable>().OrderByDescending(f => f.NumSeats)
                    .FirstOrDefault());

            ownerProperty = Create<Person>(() => Owner);
            ownerAgeProperty =
                ownerAgeProperty = Create(() => OwnerAge, () => Owner == null ? 0 : Owner.Age);

        }
    }

}
