﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Agdur;
using xReactor.Common;

namespace xReactor.Tests.Benchmarks
{
    class Program
    {
        static int defaultNumTimes = 1000;

        static void Main(string[] args)
        {
            Console.WriteLine("This may take a long time to finish. ");
            Console.WriteLine("Got bored? Well, there's one thing you can do:");
            Console.WriteLine("Optimize!");

            BenchmarkReactiveObjectCreation();
            BenchmarkCollectionItemsAdded();
            BenchmarkCollectionItemPropertySetter();
        }

        static void BenchmarkReactiveObjectCreation()
        {
            Console.WriteLine("-- rx object creation -- ");
            Console.ForegroundColor = ConsoleColor.Green;
            Benchmark.This(() => new Room()).Times(defaultNumTimes).
                Average().InMilliseconds().ToConsole().AsFormattedString();
            Console.ResetColor();
        }

        static void BenchmarkCollectionItemsAdded()
        {
            Console.WriteLine("-- collection: adding items -- ");
            Console.ForegroundColor = ConsoleColor.Green;
            Benchmark.This(() =>
            {
                var room = new Room();

                for (int index = 0; index < 100; index++)
                {
                    room.Furniture.Add(new Sofa());
                }
            }).Times(defaultNumTimes).
                Average().InMilliseconds().ToConsole().AsFormattedString();
            Console.ResetColor();
        }

        static void BenchmarkCollectionItemPropertySetter()
        {
            Console.WriteLine("-- collection: item property setter -- ");
            var room = new Room();
            var chair = new Chair();

            for (int index = 0; index < 100; index++)
            {
                room.Furniture.Add(new Sofa());
            }
            room.Furniture.Add(chair);

            Console.ForegroundColor = ConsoleColor.Green;
            Benchmark.This(() =>
            {
                chair.NumSeats = 10;
                chair.NumSeats = 20;
            }).Times(defaultNumTimes).
                Average().InMilliseconds().ToConsole().AsFormattedString();
            Console.ResetColor();
        }
    }
}
