﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reactive.Linq;
using System.Text;
using System.Threading.Tasks;

namespace xReactor
{
    public interface IProperty
    {
        Reactor Reactor { get; }
        Type Type { get; }
        object Value { get; }
        string Name { get; }
    }

    public class Property<T> : IProperty
    {
        T backingValue;

        /// <summary>
        /// Gets the name of the property.
        /// </summary>
        /// <value>The name of the property.</value>
        public string Name
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the owner of this property.
        /// </summary>
        /// <value>The owner of this property.</value>
        public Reactor Reactor
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets or sets the property value.
        /// </summary>
        /// <value>The property value.</value>
        public T Value
        {
            get { return backingValue; }
            set
            {
                //if (backingValue != value)
                if (!object.Equals(backingValue, value))
                {
                    T oldValue = backingValue;
                    RaisePropertyChangeNotifications(oldValue, backingValue = value);
                }
            }
        }

        public Type Type
        {
            get { return typeof(T); }
        }

        object IProperty.Value
        {
            get { return this.Value; }
        }

        public Property(Reactor owner, string name)
        {
            if (owner == null)
                throw new ArgumentNullException("owner");

            this.Reactor = owner;
            this.Name = name;
        }

        private void RaisePropertyChangeNotifications(T oldValue, T newValue)
        {
            Reactor.RaisePropertyChanged(this, oldValue, newValue);
        }

        public IObservable<T> GetStream()
        {
            return Observable.FromEvent<IProperty>(
                h => Reactor.RxPropertyChanged += h,
                h => Reactor.RxPropertyChanged -= h).
                Where(prop => prop == this).Select(prop => (T)prop.Value);
        }
    }
}
