﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace xReactor
{
    public class ReactiveBase : NotifyPropertyChangedBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="T:ReactiveBase"/> class.
        /// </summary>
        public ReactiveBase()
        {
            reactor = new Reactor(this, RaisePropertyChanged, null);
        }

        #region Creation methods / Reactive properties stuff

        private Reactor reactor;

        protected Property<T> Create<T>(string name, T defaultValue = default(T))
        {
            return reactor.Create<T>(name, defaultValue);
        }

        protected Property<T> Create<T>(string name, Expression<Func<T>> valueExpression)
        {
            return reactor.Create<T>(name, valueExpression);
        }

        protected Property<T> Create<T>(Expression<Func<T>> nameExpression, T defaultValue = default(T))
        {
            string propertyName = ExpressionHelper.GetNameFromExpression(nameExpression);
            return Create<T>(propertyName, defaultValue);
        }

        protected Property<T> Create<T>(Expression<Func<T>> nameExpression, Expression<Func<T>> valueExpression)
        {
            string propertyName = ExpressionHelper.GetNameFromExpression(nameExpression);
            return Create<T>(propertyName, valueExpression);
        }

        #endregion
    }
}
